local Layout = FlexBar2:NewModule("Layout");

function Layout:OnEnable()
	local SlashCommands = FlexBar2.SlashCommands;
	local ButtonList = FlexBar2.Buttons;

	local function SlashCommand_Show(Params)
		FlexBar2:Debug(Params);
		local ButtonTable = FlexBar2:ToButtonTable(Params.button);
		FlexBar2:ApplySetting(ButtonTable, "Show", true);
		for _, ButtonName in ipairs(ButtonTable) do
			ButtonList[ButtonName]:Show();
		end
	end

	local function SlashCommand_Hide(Params)
		local ButtonTable = FlexBar2:ToButtonTable(Params.button);
		FlexBar2:ApplySetting(ButtonTable, "Show", false);
		for _, ButtonName in ipairs(ButtonTable) do
			ButtonList[ButtonName]:Hide();
		end
	end
	local function SlashCommand_SetPosition(Params)
		local ButtonTable = FlexBar2:ToButtonTable(Params.button);
		if(Params.smart == "false") then
			FlexBar2:ApplySetting(ButtonTable, "X", Params.x);
			FlexBar2:ApplySetting(ButtonTable, "Y", Params.y);
		elseif(Params.smart == "true") then
			for Number, Button in ipairs(ButtonTable) do
				FlexBar2:ApplySetting(Button, "X", Params.x + (38 * (Number - ((math.ceil(Number / 12) - 1) * 12))));
				FlexBar2:ApplySetting(Button, "Y", Params.y + (38 * (math.ceil(Number / 12))));
			end
		end
		for _, ButtonName in ipairs(ButtonTable) do
			ButtonList[ButtonName]:SetPosition();
		end
	end

	SlashCommands:AddCommand("show", SlashCommand_Show, "layout", "/FlexBar2 show button=<Button>");
		SlashCommands:AddParam("show", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end)
	SlashCommands:AddCommand("hide", SlashCommand_Hide, "layout", "/FlexBar2 hide button=<Button>");
		SlashCommands:AddParam("hide", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end)
	SlashCommands:AddCommand("setposition", SlashCommand_SetPosition, "layout", "/FlexBar2 setposition button=<Button> x=<x coordinate> y=<y coordinate>");
		SlashCommands:AddParam("setposition", "button", true, nil, nil, function(Button) return FlexBar2:ButtonsExist(Button); end)
		SlashCommands:AddParam("setposition", "x", false, nil, "300", nil);
		SlashCommands:AddParam("setposition", "y", false, nil, "300", nil);
		SlashCommands:AddParam("setposition", "smart", false, {"true", "false"}, "true", nil);
end

Layout.ButtonMixin = FlexBar2.OO.Mixin
	{
		"SetPosition",
		"Show",
		"Hide",
	};

function Layout.ButtonMixin:Activate()
	FlexBar2:Debug("               Setting up layout");
	--Actual button creation (note: not setting a NormalTexture here, an apart function does that which is called later on [at the icon creation part])
	local Frame = CreateFrame("CheckButton", "FB2_BTN_" .. self.uid, UIParent, "SecureActionButtonTemplate");
	self.Frame = Frame
	
	-- Strata and sizing, drag registering
	Frame:SetFrameStrata("MEDIUM");
	Frame:SetWidth(36);
	Frame:SetHeight(36);
	Frame:RegisterForDrag("LeftButton");

	-- Textures setup
	Frame:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress");
	Frame:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight");
	Frame:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
	Frame:GetHighlightTexture():SetBlendMode("ADD");
	Frame:GetCheckedTexture():SetBlendMode("ADD");

	-- Icon setup
	Frame.Icon = self.Frame:CreateTexture(nil, "BACKGROUND")
	Frame.Icon:SetAllPoints(Frame);

	-- Cooldown ticker setup
	Frame.Cooldown = CreateFrame("Cooldown", nil, self.Frame, "CooldownFrameTemplate");
	self.LastCooldown = {};

	-- NormalTexture setup
	self.Frame:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	local NormalTexture = self.Frame:GetNormalTexture();
	NormalTexture:SetWidth(60);
	NormalTexture:SetHeight(60);
	NormalTexture:ClearAllPoints();
	NormalTexture:SetPoint("CENTER", 1, -1);

	-- Button settings
	Frame:RegisterForClicks("AnyUp");
	Frame:EnableMouse(true);
	Frame:SetClampedToScreen(true);
	Frame:SetMovable(true);
	
	self:LoadSettings();
end

function Layout.ButtonMixin:Deactivate()
	self:Hide();
end

function Layout.ButtonMixin:SetDefaults()
	self.InfoTable.Show = true;
	self.InfoTable.X = 300;
	self.InfoTable.Y = 300;
end

function Layout.ButtonMixin:LoadSettings()
	FlexBar2:Debug("loadsettings", self.InfoTable.X, ",", self.InfoTable.Y, ",", self.InfoTable.Show);
	if(type(self.InfoTable.X) == "number" and type(self.InfoTable.Y) == "number") then
		self:SetPosition(self.InfoTable.X, self.InfoTable.Y);
	else
		self:SetPosition(300, 300);
	end
	if(self.InfoTable.Show == true) then
		self:Show();
	else
		self:Hide();
	end
end

function Layout.ButtonMixin:SetPosition(X, Y, RelativeTo) -- Note: When using relativeto you should only pass frames that are anchored TOPLEFT and to UIParent, might fix that later but its enough for now
	if(type(X) ~= "number") then X = self.InfoTable.X end
	if(type(Y) ~= "number") then Y = self.InfoTable.Y end
	if(type(RelativeTo) == "table") then
		local _, rX, rY;
		if(type(RelativeTo.Frame) == "table" and type(RelativeTo.Frame.GetPoint) == "function") then
			 _, _ , _, rX, rY = RelativeTo.Frame:GetPoint();
		elseif(type(RelativeTo.GetPoint) == "function") then
			_, _ , _, rX, rY = RelativeTo:GetPoint();
		end
		self.Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", rX + X, rY - Y);
	else
		self.Frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", X, -Y);
	end	
end

function Layout.ButtonMixin:Show()
	self.Frame:Show();
end

function Layout.ButtonMixin:Hide()
	self.Frame:Hide();
end

-- GUI --
local Waterfall = FlexBar2.Waterfall;
local Positioning_X, Positioning_Y, Show, Status_Show, Positioning_AutoAlign;

function Layout.ButtonWaterfall(Action, ButtonName, ButtonTable)
	FlexBar2:Debug(Action, ButtonName, ButtonTable);
	if(Action == "edit") then
		Status_Show = FlexBar2:GetSetting(ButtonTable, "Show");
		Waterfall:AddControl(
		"type","checkbox",
		"text","Show button",
		"width",300,
		"causeRefresh", false,
		"getFunc" ,function() return Status_Show; end,
		"setFunc", function(Show) Status_Show = Show; end);
		Waterfall:AddControl(
		"type", "label",
		"text", "Button Position");
		Waterfall:AddControl(
		"type", "label",
		"text", "X: ");
		Waterfall:AddControl(
		"type", "textbox",
		"width", "50",
		"noNewLine", true,
		"causeRefresh", false,		
		"changedFunc", function(X) Positioning_X = X; end,
		"getFunc", function() return FlexBar2:GetSetting(ButtonTable, "X") or 300; end);
		Waterfall:AddControl(
		"type", "label",
		"text", "Y: ");
		Waterfall:AddControl(
		"type", "textbox",
		"width", "50",
		"noNewLine", true,
		"causeRefresh", false,		
		"changedFunc", function(Y) Positioning_Y = Y; end,
		"getFunc", function() return FlexBar2:GetSetting(ButtonTable, "X") or 300; end);
		if(#ButtonTable >= 2) then
			Waterfall:AddControl(
			"type","checkbox",
			"text","AutoAlign",
			"width",300,
			"causeRefresh", false,
			"getFunc" ,function() return Positioning_AutoAllign; end,
			"setFunc", function(AutoAlign) Positioning_AutoAlign = AutoAlign; end);
		end
	elseif(Action == "save") then
		FlexBar2:Debug("saving", ButtonTable);
		if(Positioning_AutoAlign == true) then
			for Number, Button in ipairs(ButtonTable) do
				FlexBar2:ApplySetting(Button, "X", Positioning_X + (38 * (Number - ((math.ceil(Number / 12) - 1) * 12))));
				FlexBar2:ApplySetting(Button, "Y", Positioning_Y + (38 * (math.ceil(Number / 12))));
			end
		else
			FlexBar2:ApplySetting(ButtonTable, "X", Positioning_X);
			FlexBar2:ApplySetting(ButtonTable, "Y", Positioning_Y);
		end
		FlexBar2:ApplySetting(ButtonTable, "Show", Status_Show);
	end
end

